@@ -135,7 +135,7 @@ class StompClient(LineOnlyReceiver):
         """Send connect command
         """
         self.log.debug("Sending connect command")
-        cmd = stomper.connect(self.factory.getLogin(), self.factory.getPasscode())
+        cmd = stomper.connect(self.factory.login, self.factory.passcode)
         # self.log.debug("Writing cmd: %s" % cmd)
         self.transport.write(cmd)
 
@@ -202,16 +202,23 @@ class StompClient(LineOnlyReceiver):
     
     def messageHandlerFailed(self, failure, messageId, msg, errDest):
         self.log.error("Error in message handler: %s" % str(failure))
+        disconnect = False
         #Forward message to error queue if configured
         if errDest is not None:
             errMsg = cloneStompMessageForErrorDest(msg)
             self.send(errDest, errMsg['body'], errMsg['headers'])
             self._ack(messageId)
-        #Set disconnect error
-        self.disconnectError = failure
-        #Disconnect
-        self.disconnect()
-        return failure
+            if self.factory.alwaysDisconnectOnUnhandledMsg:
+                disconnect = True
+        else:
+            disconnect = True
+        if disconnect:
+            #Set disconnect error
+            self.disconnectError = failure
+            #Disconnect
+            self.disconnect()
+            return failure
+        return None
         
     def connectTimeout(self, timeout):
         self.log.error("Connect command timed out after %s seconds" % timeout)
@@ -312,7 +319,7 @@ class StompClient(LineOnlyReceiver):
     def subscribe(self, dest, handler, headers={}, **kwargs):
         """Subscribe to a destination and register a function handler to receive messages for that destination
         """
-        errorDestination = kwargs.get('errorDestination', None);
+        errorDestination = kwargs.get('errorDestination', None)
         # client-individual mode is only supported in AMQ >= 5.2
         # headers['ack'] = headers.get('ack', 'client-individual')
         headers['ack'] = headers.get('ack', 'client')
@@ -341,17 +348,12 @@ class StompClientFactory(ClientFactory):
     protocol = StompClient
 
     def __init__(self, **kwargs):
-        self.login = kwargs.get('login', '');
-        self.passcode = kwargs.get('passcode', '');
+        self.login = kwargs.get('login', '')
+        self.passcode = kwargs.get('passcode', '')
+        self.alwaysDisconnectOnUnhandledMsg = kwargs.get('alwaysDisconnectOnUnhandledMsg', True)
         self.buildProtocolDeferred = defer.Deferred()
         self.log = logging.getLogger(LOG_CATEGORY)
     
-    def getLogin(self):
-        return self.login
-
-    def getPasscode(self):
-        return self.passcode
-
     def buildProtocol(self, addr):
         p = ClientFactory.buildProtocol(self, addr)
         #This is a sneaky way of passing the protocol instance back to the caller
@@ -369,15 +371,16 @@ class StompConfig(object):
     def __init__(self, host, port, **kwargs):
         self.host = host
         self.port = port
-        self.login = kwargs.get('login', '');
-        self.passcode = kwargs.get('passcode', '');
+        self.login = kwargs.get('login', '')
+        self.passcode = kwargs.get('passcode', '')
         self.log = logging.getLogger(LOG_CATEGORY)
 
 class StompCreator(object):
 
     def __init__(self, config, **kwargs):
         self.config = config
-        self.connectTimeout = kwargs.get('connectTimeout', None);
+        self.connectTimeout = kwargs.get('connectTimeout', None)
+        self.alwaysDisconnectOnUnhandledMsg = kwargs.get('alwaysDisconnectOnUnhandledMsg', False)
         self.log = logging.getLogger(LOG_CATEGORY)
         self.stompConnectedDeferred = None
 
@@ -387,7 +390,7 @@ class StompCreator(object):
         return self.stompConnectedDeferred
 
     def connect(self):
-        factory = StompClientFactory(login=self.config.login, passcode=self.config.passcode)
+        factory = StompClientFactory(login=self.config.login, passcode=self.config.passcode, alwaysDisconnectOnUnhandledMsg=self.alwaysDisconnectOnUnhandledMsg)
         reactor.connectTCP(self.config.host, self.config.port, factory) 
         return factory.buildProtocolDeferred
         
