@@ -57,9 +57,6 @@ class HandlerExceptionWithErrorQueueIntegrationTestCase(unittest.TestCase):
     msg2 = 'follow up message'
     queue = '/queue/asyncStompestHandlerExceptionWithErrorQueueUnitTest'
     errorQueue = '/queue/zzz.error.asyncStompestHandlerExceptionWithErrorQueueUnitTest'
-    disconnected = defer.Deferred()
-    disconnectedAgain = defer.Deferred()
-    disconnectedFromErrQ = defer.Deferred()
     ackMode = getClientAckMode()
         
     def cleanQueues(self):
@@ -74,92 +71,187 @@ class HandlerExceptionWithErrorQueueIntegrationTestCase(unittest.TestCase):
             frame = stomp.receiveFrame()
             stomp.ack(frame)
             print "Dequeued old message: %s" % frame
-        stomp.disconnect()        
-    
-    def test_onhandlerException_ackMessage_filterReservedHdrs_send2ErrorQ_and_errback(self):
-        self.cleanQueues()
-        self.config = StompConfig('localhost', 61613)
-        self.creator = StompCreator(self.config)
-        deferConnected = self.creator.getConnection().addCallback(self._onConnected)
-        deferredTestErr = self.assertFailure(self.disconnected, StompestTestError).addCallback(self._reconnectAfterErr)
-        self.disconnectedAgain.addCallback(self._reconnectForErrQ)
-        self.disconnectedFromErrQ.addCallback(self._verifyErrMsg) #asserts err message content is correct
-        return defer.DeferredList([
-            deferredTestErr, #asserts that handler exception triggered disconnect and was propagated to disconnect deferred
-            self.disconnectedAgain, #asserts that message causing the exception was ack'ed to broker (otherwise, this message would not get delivered)
-            self.disconnectedFromErrQ, #asserts that we were able to read a message from the error queue
-            ]
-        )
+        stomp.disconnect()
         
-    def _onConnected(self, stomp):
-        stomp.getDisconnectedDeferred().addCallback(self.disconnected.callback).addErrback(self.disconnected.errback)
+    def setUp(self):
+        self.unhandledMsg = None
+        self.errQMsg = None
+        self.consumedMsg = None
+        self.msgsHandled = 0
+
+    @defer.inlineCallbacks
+    def test_onhandlerException_ackMessage_filterReservedHdrs_send2ErrorQ_and_disconnect(self):
+        self.cleanQueues()
+        
+        config = StompConfig('localhost', 61613)
+        creator = StompCreator(config, alwaysDisconnectOnUnhandledMsg=True)
+        
+        #Connect
+        stomp = yield creator.getConnection()
+        
+        #Enqueue two messages
         stomp.send(self.queue, self.msg1, self.msg1Hdrs)
         stomp.send(self.queue, self.msg2)
+        
+        #Barf on first message so it will get put in error queue
         #Use selector to guarantee message order.  AMQ doesn't not guarantee order by default
         stomp.subscribe(self.queue, self._saveMsgAndBarf, {'ack': self.ackMode, 'activemq.prefetchSize': 1, 'selector': "food = 'barf'"}, errorDestination=self.errorQueue)
         
+        #Client disconnected and returned error
+        try:
+            yield stomp.getDisconnectedDeferred()
+        except StompestTestError, e:
+            pass
+        else:
+            self.assertTrue(False)
+        
+        #Reconnect and subscribe again - consuming second message then disconnecting
+        stomp = yield creator.getConnection()
+        stomp.subscribe(self.queue, self._eatOneMsgAndDisconnect, {'ack': self.ackMode, 'activemq.prefetchSize': 1}, errorDestination=self.errorQueue)
+        
+        #Client disconnects without error
+        yield stomp.getDisconnectedDeferred()
+        
+        #Reconnect and subscribe to error queue
+        stomp = yield creator.getConnection()
+        stomp.subscribe(self.errorQueue, self._saveErrMsgAndDisconnect, {'ack': self.ackMode, 'activemq.prefetchSize': 1})
+        
+        #Wait for disconnect
+        yield stomp.getDisconnectedDeferred()
+        
+        #Verify that first message was in error queue
+        self.assertEquals(self.msg1, self.errQMsg['body'])
+        self.assertEquals(self.msg1Hdrs['food'], self.errQMsg['headers']['food'])
+        self.assertNotEquals(self.unhandledMsg['headers']['message-id'], self.errQMsg['headers']['message-id'])
+        
+        #Verify that second message was consumed
+        self.assertEquals(self.msg2, self.consumedMsg['body'])
+
+    @defer.inlineCallbacks
+    def test_onhandlerException_ackMessage_filterReservedHdrs_send2ErrorQ_and_no_disconnect(self):
+        self.cleanQueues()
+        
+        config = StompConfig('localhost', 61613)
+        creator = StompCreator(config)
+        
+        #Connect
+        stomp = yield creator.getConnection()
+        
+        #Enqueue two messages
+        stomp.send(self.queue, self.msg1, self.msg1Hdrs)
+        stomp.send(self.queue, self.msg2)
+        
+        #Barf on first msg, disconnect on second msg
+        stomp.subscribe(self.queue, self._barfOneEatOneAndDisonnect, {'ack': self.ackMode, 'activemq.prefetchSize': 1}, errorDestination=self.errorQueue)
+        
+        #Client disconnects without error
+        yield stomp.getDisconnectedDeferred()
+        
+        #Reconnect and subscribe to error queue
+        stomp = yield creator.getConnection()
+        stomp.subscribe(self.errorQueue, self._saveErrMsgAndDisconnect, {'ack': self.ackMode, 'activemq.prefetchSize': 1})
+        
+        #Wait for disconnect
+        yield stomp.getDisconnectedDeferred()
+        
+        #Verify that one message was in error queue (can't guarantee order)
+        self.assertNotEquals(None, self.errQMsg)
+        self.assertTrue(self.errQMsg['body'] in (self.msg1, self.msg2))
+
+    @defer.inlineCallbacks
+    def test_onhandlerException_disconnect(self):
+        self.cleanQueues()
+        
+        config = StompConfig('localhost', 61613)
+        creator = StompCreator(config)
+        
+        #Connect
+        stomp = yield creator.getConnection()
+        
+        #Enqueue a message
+        stomp.send(self.queue, self.msg1, self.msg1Hdrs)
+        
+        #Barf on first msg (implicit disconnect)
+        stomp.subscribe(self.queue, self._saveMsgAndBarf, {'ack': self.ackMode, 'activemq.prefetchSize': 1})
+        
+        #Client disconnected and returned error
+        try:
+            yield stomp.getDisconnectedDeferred()
+        except StompestTestError, e:
+            pass
+        else:
+            self.assertTrue(False)
+            
+        #Reconnect and subscribe again - consuming retried message and disconnecting
+        stomp = yield creator.getConnection()
+        stomp.subscribe(self.queue, self._eatOneMsgAndDisconnect, {'ack': self.ackMode, 'activemq.prefetchSize': 1})
+        
+        #Client disconnects without error
+        yield stomp.getDisconnectedDeferred()
+        
+        #Verify that message was retried
+        self.assertEquals(self.msg1, self.unhandledMsg['body'])
+        self.assertEquals(self.msg1, self.consumedMsg['body'])
+        self.assertEquals(self.unhandledMsg['headers']['message-id'], self.consumedMsg['headers']['message-id'])
+
     def _saveMsgAndBarf(self, stomp, msg):
         print 'Save message and barf'
-        self.assertEquals(self.msg1, msg['body'])
         self.unhandledMsg = msg
         raise StompestTestError('this is a test')
         
-    def _reconnectAfterErr(self, result):
-        self.creator.getConnection().addCallback(self._onConnectedAgain)
-        
-    def _onConnectedAgain(self, stomp):
-        stomp.getDisconnectedDeferred().addCallback(self.disconnectedAgain.callback).addErrback(self.disconnectedAgain.errback)
-        stomp.subscribe(self.queue, self._eatOneMsgAndDisconnect, {'ack': self.ackMode, 'activemq.prefetchSize': 1}, errorDestination=self.errorQueue)
+    def _barfOneEatOneAndDisonnect(self, stomp, msg):
+        self.msgsHandled += 1
+        if self.msgsHandled == 1:
+            self._saveMsgAndBarf(stomp, msg)
+        self._eatOneMsgAndDisconnect(stomp, msg)
         
     def _eatOneMsgAndDisconnect(self, stomp, msg):
         print 'Eat message and disconnect'
-        self.assertEquals(self.msg2, msg['body'])
+        self.consumedMsg = msg
         stomp.disconnect()
-        
-    def _reconnectForErrQ(self, result):
-        self.creator.getConnection().addCallback(self._onConnectedForErrQ)
-        
-    def _onConnectedForErrQ(self, stomp):
-        stomp.getDisconnectedDeferred().addCallback(self.disconnectedFromErrQ.callback).addErrback(self.disconnectedFromErrQ.errback)
-        stomp.subscribe(self.errorQueue, self._saveErrMsgAndDisconnect, {'ack': self.ackMode, 'activemq.prefetchSize': 1})
-        
+                
     def _saveErrMsgAndDisconnect(self, stomp, msg):
         print 'Save error message and disconnect'
         self.errQMsg = msg
         stomp.disconnect()
-        
-    def _verifyErrMsg(self, result):
-        self.assertEquals(self.msg1, self.errQMsg['body'])
-        self.assertEquals(self.msg1Hdrs['food'], self.errQMsg['headers']['food'])
-        self.assertNotEquals(self.unhandledMsg['headers']['message-id'], self.errQMsg['headers']['message-id'])
 
 class GracefulDisconnectTestCase(unittest.TestCase):
     numMsgs = 5
     msgCount = 0
     msg = 'test'
     queue = '/queue/aysncStompestGracefulDisconnectUnitTest'
-    disconnected = defer.Deferred()
-    disconnectedAgain = defer.Deferred()
     ackMode = getClientAckMode()
     
+    @defer.inlineCallbacks
     def test_onDisconnect_waitForOutstandingMessagesToFinish(self):
         self.config = StompConfig('localhost', 61613)
         self.creator = StompCreator(self.config)
-        deferConnected = self.creator.getConnection().addCallback(self._onConnected)
-        self.disconnected.addCallback(self._reconnect)
-        self.disconnectedAgain.addCallback(self._verifyAllMsgsAcked)
-        return defer.DeferredList([
-                self.disconnected,
-                self.disconnectedAgain,
-            ]
-        )
         
-    def _onConnected(self, stomp):
-        stomp.getDisconnectedDeferred().addCallback(self.disconnected.callback).addErrback(self.disconnected.errback)
+        config = StompConfig('localhost', 61613)
+        creator = StompCreator(config)
+        
+        #Connect
+        stomp = yield creator.getConnection()
+        
         for i in range(self.numMsgs):
             stomp.send(self.queue, self.msg)
         stomp.subscribe(self.queue, self._msgHandler, {'ack': self.ackMode, 'activemq.prefetchSize': self.numMsgs})
         
+        #Wait for disconnect
+        yield stomp.getDisconnectedDeferred()
+        
+        #Reconnect and subscribe again to make sure that all messages in the queue were ack'ed
+        stomp = yield creator.getConnection()
+        self.timeExpired = False
+        self.timeoutDelayedCall = reactor.callLater(1, self._timesUp, stomp)
+        stomp.subscribe(self.queue, self._eatOneMsgAndDisconnect, {'ack': self.ackMode, 'activemq.prefetchSize': self.numMsgs})
+
+        #Wait for disconnect
+        yield stomp.getDisconnectedDeferred()
+        
+        #Time should have expired if there were no messages left in the queue
+        self.assertTrue(self.timeExpired)
+                
     def _msgHandler(self, stomp, msg):
         self.msgCount += 1
         if self.msgCount < self.numMsgs:
@@ -169,15 +261,6 @@ class GracefulDisconnectTestCase(unittest.TestCase):
         else:
             stomp.disconnect()
             
-    def _reconnect(self, result):
-        self.creator.getConnection().addCallback(self._onReconnect)
-        
-    def _onReconnect(self, stomp):
-        stomp.getDisconnectedDeferred().addCallback(self.disconnectedAgain.callback).addErrback(self.disconnectedAgain.errback)
-        self.timeExpired = False
-        self.timeoutDelayedCall = reactor.callLater(1, self._timesUp, stomp)
-        stomp.subscribe(self.queue, self._eatOneMsgAndDisconnect, {'ack': self.ackMode, 'activemq.prefetchSize': self.numMsgs})
-
     def _timesUp(self, stomp):
         print "Times up!!!"
         self.timeExpired = True
@@ -186,11 +269,6 @@ class GracefulDisconnectTestCase(unittest.TestCase):
     def _eatOneMsgAndDisconnect(self, stomp, msg):
         self.timeoutDelayedCall.cancel()
         stomp.disconnect()
-        
-    def _verifyAllMsgsAcked(self, result):
-        print "VERIFY!"
-        self.assertTrue(self.timeExpired)
-        return result
 
 if __name__ == '__main__':
     import sys
